--[[---------------------------------------------------------------------------\

    Credit to Minotorious for developing a working gatherable behavior tree

\---------------------------------------------------------------------------]]--

local mod = ...



-- Node: Place stump --------------------------------------------------------------------------------------------------------
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_NODE_PLACE_STUMP",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
		ResourcePosition = "BEHAVIOR_TREE_DATA_LOCATION",
		ResourceObject = "BEHAVIOR_TREE_DATA_VOID_OBJECT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent

        local objResource = nil
        objResource = self.ResourcePosition.DestinationObject

        local posResource = nil
        posResource = self.ResourcePosition.DestinationPosition

        -- Create the stump
        local objStump = nil
        local treeIndex = math.random(1, 3)
        objStump = compAgent:getLevel():createObject("PREFAB_STUMP_STUMP" .. treeIndex, posResource)

        if (objStump) then

            local scaleGirth = math.random(50, 100) / 100
            local scaleHeight = math.random(85, 110) / 100
            objStump:scale({ scaleGirth, scaleHeight, scaleGirth })

            local orient = 2 * math.pi * math.random()
            objStump:setRotationY(orient)

        end

        return BEHAVIOR_TREE_NODE_RESULT.TRUE

    end -- update

})



---------------------------------------------------------------------------------------------------
-- Behavior Tree ----------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
mod:registerBehaviorTree({
	Id = "BEHAVIOR_WOODCUTTER_STUMP",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {}
		},
        {
			Name = "GatheringData",
			DataType = "BEHAVIOR_TREE_DATA_GATHERING",
			IsPublic = false,
			InitialValue = {}
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.GATHER,
				SetIdleAfterWait = false
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "ResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "AroundResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "ShouldLookAtResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = true
			}
		},
        {
			Name = "ShouldReceiveXp",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = false
			}
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 1,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false
			}
		},
	},
	Root = {
        Name = "WorkLoopRepeater",
        Type = "REPEAT",
        RepeatData = "WorkLoop",
        Child = {
            Name = "WorkLoopSequencer",
            Type = "SEQUENCER",
            Children = {
                {
                    Name = "SetupGatheringWork",
                    Type = "SETUP_GATHERING_WORK",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData",
                    StepsCountLoopData = "WorkLoop",
                    GatherWaitData = "DoJobTimer"
                },
                {
                    Name = "SetWorkplaceDestination",
                    Type = "SET_WORKPLACE_AS_DESTINATION",
                    AgentData = "AgentData",
                    WorkplacePosition = "WorkplacePosition",
                },
                {
                    Name = "GoToWorkplace",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "GatherRequest",
                    Type = "GATHER_REQUEST",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData",
                    GatherInOneStepLoopData = "WorkLoop"
                },
                {
                    Name = "FetchProduce",
                    Type = "FETCH_NEXT_GATHERABLE",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData",
                    TimeToWait = "DoJobTimer",
                    ResourcePosition = "ResourcePosition",
                    AroundResourcePosition = "AroundResourcePosition",
                    ShouldLookAtResourcePosition = "ShouldLookAtResourcePosition"
                },
                {
                    Name = "DisablePaths",
                    Type = "DISABLE_PATH_TRACING",
                    AgentData = "AgentData"
                },
                {
                    Name = "GoToGatherable",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "ResourcePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "LookAtGatherable",
                    Type = "LOOK_AT",
                    AgentData = "AgentData",
                    Destination = "ResourcePosition"
                },
                {
                    Name = "GatherResource",
                    Type = "GATHER_RESOURCE",
                    AgentData = "AgentData",
                    TimeToWait = "DoJobTimer",
                    Gather = "GatheringData",
                    ShouldReceiveXp = "ShouldReceiveXp"
                },
                {
                    Name = "PlaceStump",
                    Type = "BEHAVIOR_NODE_PLACE_STUMP",
                    AgentData = "AgentData",
                    ResourcePosition = "ResourcePosition",
                },
                {
                    Name = "GoToWorkplaceFin",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "EnablePaths",
                    Type = "ENABLE_PATH_TRACING",
                    AgentData = "AgentData"
                },
                {
                    Name = "AddToInventory",
                    Type = "ADD_TO_INVENTORY",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData"
                },
                {
                    Name = "GiveXp",
                    Type = "GIVE_JOB_XP",
                    AgentData = "AgentData",
                    ShouldReceiveXp = "ShouldReceiveXp"
                }
            }
        }
	}
})